import sys
import os
import log
from dotenv import dotenv_values
from wowipy.wowipy import WowiPy
import sqlalchemy.exc
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from wowicache.models import Base, EconomicUnit, District, Building, UseUnit, Address, Communication, Person, Contract
from wowicache.models import Contractor
from datetime import datetime

ENBUILDINGS = 1
ENCONTRACTORS = 2
ENPERSONS = 3
ENECONOMICUNITS = 4
ENCONTRACTS = 5
ENUSEUNITS = 6


def handle_unhandled_exception(exc_type, exc_value, exc_traceback):
    if issubclass(exc_type, KeyboardInterrupt):
        sys.__excepthook__(exc_type, exc_value, exc_traceback)
        return
    logger.critical("Unhandled exception", exc_info=(exc_type, exc_value, exc_traceback))


if len(sys.argv) > 1:
    settings_path = sys.argv[1]
else:
    settings_path = os.path.join(os.path.abspath(os.path.dirname(__file__)), ".env")
settings = dotenv_values(settings_path)
logger = log.setup_custom_logger('root', settings.get("log_method", "file"),
                                 settings.get("log_level", "info"),
                                 graylog_host=settings.get("graylog_server"),
                                 graylog_port=int(settings.get("graylog_port")))
sys.excepthook = handle_unhandled_exception


def cache_to_db():
    logger.info("cache_to_db started.")

    wowi_host = settings.get("wowi_host")
    wowi_user = settings.get("wowi_user")
    wowi_pass = settings.get("wowi_pass")
    wowi_key = settings.get("wowi_key")

    connection_string = settings.get("db_connection_string")

    str_en_buildings = settings.get("enable_buildings")
    str_en_contractors = settings.get("enable_contractors")
    str_en_persons = settings.get("enable_persons")
    str_en_economic_units = settings.get("enable_economic_units")
    str_en_license_agreements = settings.get("enable_license_agreements")
    str_en_use_units = settings.get("enable_use_units")

    user_agent = settings.get("user_agent")
    if user_agent is None or len(user_agent.strip()) == 0:
        user_agent = "WowiCache/1.0"

    entities = []
    if str_en_buildings is not None and str_en_buildings.lower() == "true":
        entities.append(ENBUILDINGS)
    if str_en_contractors is not None and str_en_contractors.lower() == "true":
        entities.append(ENCONTRACTORS)
    if str_en_persons is not None and str_en_persons.lower() == "true":
        entities.append(ENPERSONS)
    if str_en_economic_units is not None and str_en_economic_units.lower() == "true":
        entities.append(ENECONOMICUNITS)
    if str_en_license_agreements is not None and str_en_license_agreements.lower() == "true":
        entities.append(ENCONTRACTS)
    if str_en_use_units is not None and str_en_use_units.lower() == "true":
        entities.append(ENUSEUNITS)

    logger.info(f"Entities activated: {entities}")

    wowicon = WowiPy(hostname=wowi_host, user=wowi_user,
                     password=wowi_pass, api_key=wowi_key,
                     user_agent=user_agent)

    engine = create_engine(connection_string, echo=False, pool_pre_ping=True)
    Base.metadata.create_all(bind=engine)
    Session = sessionmaker(bind=engine)
    session = Session()

    session.query(Contractor).delete()
    session.query(Contract).delete()
    session.query(Communication).delete()
    session.query(Address).delete()
    session.query(Person).delete()
    session.query(UseUnit).delete()
    session.query(Building).delete()
    session.query(EconomicUnit).delete()
    session.query(District).delete()
    session.commit()

    if ENECONOMICUNITS in entities or ENBUILDINGS in entities:
        sectioncount = 0
        districts = wowicon.get_districts()
        for entry in districts:
            new_entry = District(internal_id=entry.id_,
                                 name=entry.name)
            session.add(new_entry)
            sectioncount += 1
        session.commit()
        logger.info(f"Added {sectioncount} districts.")

    if ENECONOMICUNITS in entities:
        sectioncount = 0
        economic_units = wowicon.get_economic_units(fetch_all=True)
        for entry in economic_units:
            district_id = entry.district.id_ if entry.district else None
            new_entry = EconomicUnit(internal_id=entry.id_,
                                     id_num=entry.id_num,
                                     company_id=entry.company_code.id_,
                                     name=entry.name,
                                     location=entry.location,
                                     construction_year=entry.construction_year,
                                     info=entry.info,
                                     owner_id=entry.owner.id_,
                                     district_id=district_id)
            session.add(new_entry)
            sectioncount += 1
        session.commit()
        logger.info(f"Added {sectioncount} economic units.")

    if ENBUILDINGS in entities:
        sectioncount = 0
        buildings = wowicon.get_building_lands(fetch_all=True)
        for entry in buildings:
            district_id = entry.building.district.id_ if entry.building.district else None
            move_in_date = datetime.strptime(str(entry.building.move_in_date), "%Y-%m-%d") \
                if entry.building.move_in_date else None
            entry_date = datetime.strptime(str(entry.entry_date), "%Y-%m-%d") \
                if entry.entry_date else None
            new_entry = Building(internal_id=entry.id_,
                                 id_num=entry.id_num,
                                 company_id=entry.company_code.id_,
                                 building_land_type=entry.building_land_type,
                                 entry_date=entry_date,
                                 economic_unit_id=entry.economic_unit.id_,
                                 postcode=entry.estate_address.zip_,
                                 town=entry.estate_address.town,
                                 street=entry.estate_address.street,
                                 house_number=entry.estate_address.house_number,
                                 house_number_addition=entry.estate_address.house_number_addition,
                                 country_id=entry.estate_address.country_id,
                                 country=entry.estate_address.country_code,
                                 street_complete=entry.estate_address.street_complete,
                                 house_number_complete=entry.estate_address.house_number_complete,
                                 construction_year=entry.building.construction_year,
                                 move_in_date=move_in_date,
                                 building_type_id=entry.building.building_type.id_,
                                 building_type_name=entry.building.building_type.name,
                                 district_id=district_id)
            session.add(new_entry)
            sectioncount += 1
        session.commit()
        logger.info(f"Added {sectioncount} buildings.")

    if ENUSEUNITS in entities:
        sectioncount = 0
        use_units = wowicon.get_use_units(fetch_all=True)
        for entry in use_units:
            move_in_date = datetime.strptime(str(entry.move_in_date), "%Y-%m-%d") \
                if entry.move_in_date else None
            entry_date = datetime.strptime(str(entry.entry_date), "%Y-%m-%d") \
                if entry.entry_date else None
            exit_date = datetime.strptime(str(entry.exit_date), "%Y-%m-%d") \
                if entry.exit_date else None
            management_start = datetime.strptime(str(entry.management_start), "%Y-%m-%d") \
                if entry.management_start else None
            management_end = datetime.strptime(str(entry.management_end), "%Y-%m-%d") \
                if entry.management_end else None
            financing_type_id = entry.financing_type.id_ if entry.financing_type else None
            financing_type = entry.financing_type.name if entry.financing_type else None

            position_id = entry.position.id_ if entry.position else None
            position = entry.position.name if entry.position else None

            floor_id = entry.floor.id_ if entry.floor else None
            floor_name = entry.floor.name if entry.floor else None
            floor_level = entry.floor.level_to_ground if entry.floor else None
            new_entry = UseUnit(internal_id=entry.id_,
                                id_num=entry.id_num,
                                company_id=entry.company_code.id_,
                                entry_date=entry_date,
                                building_id=entry.building_land.id_,
                                economic_unit_id=entry.economic_unit.id_,
                                postcode=entry.estate_address.zip_,
                                town=entry.estate_address.town,
                                street=entry.estate_address.street,
                                house_number=entry.estate_address.house_number,
                                house_number_addition=entry.estate_address.house_number_addition,
                                country_id=entry.estate_address.country_id,
                                country=entry.estate_address.country_code,
                                street_complete=entry.estate_address.street_complete,
                                house_number_complete=entry.estate_address.house_number_complete,
                                financing_type_id=financing_type_id,
                                financing_type=financing_type,
                                use_unit_usage_type_id=entry.current_use_unit_type.use_unit_usage_type.id_,
                                use_unit_usage_type=entry.current_use_unit_type.use_unit_usage_type.name,
                                usable_space=entry.usable_space,
                                living_space=entry.living_space,
                                heating_space=entry.heating_space,
                                number_of_rooms=entry.number_of_rooms,
                                number_of_half_rooms=entry.number_of_half_rooms,
                                description_of_position=entry.description_of_position,
                                management_start=management_start,
                                management_end=management_end,
                                move_in_date=move_in_date,
                                exit_date=exit_date,
                                position_id=position_id,
                                position=position,
                                floor_id=floor_id,
                                floor_name=floor_name,
                                floor_level=floor_level)
            session.add(new_entry)
            sectioncount += 1
        session.commit()
        logger.info(f"Added {sectioncount} use units.")

    # Für Contractors benötigen wir auch Personen
    if ENCONTRACTORS in entities and ENPERSONS not in entities:
        entities.append(ENPERSONS)

    if ENPERSONS in entities:
        sectioncount = 0
        persons = wowicon.get_persons(fetch_all=True)
        for entry in persons:
            valid_from = datetime.strptime(str(entry.valid_from), "%Y-%m-%d") \
                if entry.valid_from else None
            valid_to = datetime.strptime(str(entry.valid_to), "%Y-%m-%d") \
                if entry.valid_to else None
            birth_date = datetime.strptime(str(entry.natural_person.birth_date), "%Y-%m-%d") \
                if entry.natural_person.birth_date else None
            gender_id = entry.natural_person.gender.id_ if entry.natural_person.gender else None
            gender_name = entry.natural_person.gender.name if entry.natural_person.gender else None

            new_person = Person(internal_id=entry.id_,
                                id_num=entry.id_num,
                                name=entry.name,
                                short_name=entry.shortname,
                                valid_from=valid_from,
                                valid_to=valid_to,
                                long_name_1=entry.legal_person.long_name1,
                                long_name_2=entry.legal_person.long_name2,
                                vat_id=entry.legal_person.vat_id,
                                commercial_register_number=entry.legal_person.commercial_register_number,
                                commercial_register_town=entry.legal_person.commercial_register_town,
                                first_name=entry.natural_person.first_name,
                                last_name=entry.natural_person.last_name,
                                birth_date=birth_date,
                                gender_id=gender_id,
                                gender_name=gender_name)
            session.add(new_person)
            sectioncount += 1

            if entry.addresses is not None:
                for address_entry in entry.addresses:
                    address_valid_from = datetime.strptime(str(address_entry.valid_from), "%Y-%m-%d") \
                        if address_entry.valid_from else None
                    address_valid_to = datetime.strptime(str(address_entry.valid_to), "%Y-%m-%d") \
                        if address_entry.valid_to else None

                    country_id = address_entry.country.id_ if address_entry.country else None
                    country = address_entry.country.code if address_entry.country else None

                    address_type_id = address_entry.address_type.id_ if address_entry.address_type else None
                    address_type = address_entry.address_type.name if address_entry.address_type else None

                    new_address = Address(internal_id=address_entry.id_,
                                          postcode=address_entry.zip_,
                                          town=address_entry.town,
                                          street=address_entry.street,
                                          house_number=address_entry.house_number,
                                          house_number_addition=address_entry.house_number_addition,
                                          country_id=country_id,
                                          country=country,
                                          street_complete=address_entry.street_complete,
                                          house_number_complete=address_entry.house_number_complete,
                                          address_type_id=address_type_id,
                                          address_type=address_type,
                                          valid_from=address_valid_from,
                                          valid_to=address_valid_to,
                                          person_id=new_person.internal_id)
                    session.add(new_address)

            if entry.communications is not None:
                for comm_entry in entry.communications:
                    type_phone = [1, 3]
                    if comm_entry.communication_type.id_ in type_phone:
                        if comm_entry.content is not None:
                            comm_entry.content = comm_entry.content.replace(' ', '').strip()
                            comm_entry.content = comm_entry.content.replace('0049', '0')
                            comm_entry.content = comm_entry.content.replace('+49', '0')

                    new_comm = Communication(internal_id=comm_entry.id_,
                                             related_address_id=comm_entry.related_address_id,
                                             content=comm_entry.content,
                                             explanation=comm_entry.explanation,
                                             communication_type_id=comm_entry.communication_type.id_,
                                             communication_type=comm_entry.communication_type.name,
                                             person_id=new_person.internal_id)
                    session.add(new_comm)
        session.commit()
        logger.info(f"Added {sectioncount} persons.")

    # Für Contractors brauchen wir auch Contracts
    if ENCONTRACTORS in entities and ENCONTRACTS not in entities:
        entities.append(ENCONTRACTS)

    if ENCONTRACTS in entities:
        sectioncount = 0
        contracts = wowicon.get_license_agreements(fetch_all=True)
        for entry in contracts:
            print(str(entry.start_contract))
            contract_start = datetime.strptime(str(entry.start_contract), "%Y-%m-%d %H:%M:%S") \
                if entry.start_contract else None
            contract_end = datetime.strptime(str(entry.end_of_contract), "%Y-%m-%d  %H:%M:%S") \
                if entry.end_of_contract else None
            new_entry = Contract(internal_id=entry.id_,
                                 id_num=entry.id_num,
                                 use_unit_id=entry.use_unit.id_,
                                 restriction_id=entry.restriction_of_use.id_,
                                 restriction_name=entry.restriction_of_use.name,
                                 is_vacancy=entry.restriction_of_use.is_vacancy,
                                 status_id=entry.status_contract.id_,
                                 status_name=entry.status_contract.name,
                                 life_id=entry.life_of_contract.id_,
                                 life_name=entry.life_of_contract.name,
                                 contract_start=contract_start,
                                 contract_end=contract_end)
            session.add(new_entry)
            sectioncount += 1
        session.commit()
        logger.info(f"Added {sectioncount} contracts.")

    if ENCONTRACTORS in entities:
        sectioncount = 0
        contractors = wowicon.get_contractors(fetch_all=True)
        for entry in contractors:
            # end_contract = datetime.strptime(str(entry.end_of_contract), "%Y-%m-%d") \
            #     if entry.end_of_contract else None
            # if end_contract is not None:
            #     print(f"ID: {entry.id_}")
            #     # print(end_contract)
            #     # print(datetime.now() > end_contract)
            #     if datetime.now() > end_contract:
            #         # print("Überspringen")
            #         continue
            valid_from = datetime.strptime(str(entry.contractual_use_valid_from), "%Y-%m-%d") \
                if entry.contractual_use_valid_from else None
            valid_to = datetime.strptime(str(entry.contractual_use_valid_to), "%Y-%m-%d") \
                if entry.contractual_use_valid_to else None

            new_entry = Contractor(internal_id=entry.id_,
                                   contract_id=entry.license_agreement_id,
                                   use_unit_id=entry.use_unit.id_,
                                   person_id=entry.person.id_,
                                   type_id=entry.contractor_type.id_,
                                   type_name=entry.contractor_type.name,
                                   valid_from=valid_from,
                                   valid_to=valid_to)
            try:
                session.add(new_entry)
                session.commit()
                sectioncount += 1
            except sqlalchemy.exc.IntegrityError:
                logger.warning(f"Rolling back contractor")
                session.rollback()

        logger.info(f"Added {sectioncount} contractors.")

    session.close()
    logger.info("Cache update complete.")


if __name__ == '__main__':
    cache_to_db()



from sqlalchemy import Column, String, Integer, ForeignKey, Date, Float, Boolean
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

Base = declarative_base()


class WowiCache:
    def __init__(self, connection_string: str):
        engine = create_engine(connection_string, echo=False, pool_pre_ping=True)
        Session = sessionmaker(bind=engine)
        self.session = Session()


class District(Base):
    __tablename__ = "wowi_districts"
    internal_id = Column("internal_id", Integer, primary_key=True)
    name = Column("name", String(50))

    economic_units = relationship('EconomicUnit', back_populates='district')
    buildings = relationship('Building', back_populates='district')

    def __init__(self, internal_id, name):
        self.internal_id = internal_id
        self.name = name

    def __repr__(self):
        return f"District {self.name} ({self.internal_id})"


class EconomicUnit(Base):
    __tablename__ = "wowi_economic_units"
    internal_id = Column("internal_id", Integer, primary_key=True)
    id_num = Column("id_num", String(50))
    company_id = Column("company_id", Integer)
    name = Column("name", String(100))
    location = Column("location", String(100))
    construction_year = Column("construction_year", Integer, nullable=True)
    info = Column("info", String(255), nullable=True)
    owner_id = Column("owner_id", Integer)
    district_id = Column(Integer, ForeignKey("wowi_districts.internal_id"), nullable=True)
    district = relationship('District', back_populates='economic_units')
    buildings = relationship('Building', back_populates='economic_unit')
    use_units = relationship('UseUnit', back_populates='economic_unit')

    def __init__(self, internal_id, id_num, company_id, name, location, construction_year,
                 info, owner_id, district_id):
        self.internal_id = internal_id
        self.id_num = id_num
        self.company_id = company_id
        self.name = name
        self.location = location
        self.construction_year = construction_year
        self.info = info
        self.owner_id = owner_id
        self.district_id = district_id

    def __repr__(self):
        return f"Economic Unit Id: {self.internal_id} IdNum: {self.id_num} Name: {self.name}"


class Building(Base):
    __tablename__ = "wowi_buildings"
    internal_id = Column("internal_id", Integer, primary_key=True)
    id_num = Column("id_num", String(50))
    company_id = Column("company_id", Integer)
    building_land_type = Column("building_land_type", String(50))
    entry_date = Column("entry_date", Date)
    economic_unit_id = Column(Integer, ForeignKey("wowi_economic_units.internal_id"))
    economic_unit = relationship('EconomicUnit', back_populates='buildings')
    postcode = Column("postcode", String(30))
    town = Column("town", String(50))
    street = Column("street", String(70))
    house_number = Column("house_number", String(30))
    house_number_addition = Column("house_number_addition", String(30), nullable=True)
    country_id = Column("country_id", Integer)
    country = Column("country", String(50))
    street_complete = Column("street_complete", String(80))
    house_number_complete = Column("house_number_complete", String(50))

    construction_year = Column("construction_year", Integer, nullable=True)
    move_in_date = Column("move_in_date", Date, nullable=True)

    building_type_id = Column("building_type_id", Integer)
    building_type_name = Column("building_type_name", String(60))

    district_id = Column(Integer, ForeignKey("wowi_districts.internal_id"), nullable=True)
    district = relationship('District', back_populates='buildings')

    use_units = relationship('UseUnit', back_populates='building')

    def __init__(self, internal_id, id_num, company_id, building_land_type, entry_date, economic_unit_id,
                 postcode, town, street, house_number, house_number_addition, country_id, country,
                 street_complete, house_number_complete, construction_year, move_in_date,
                 building_type_id, building_type_name, district_id):
        self.internal_id = internal_id
        self.id_num = id_num
        self.company_id = company_id
        self.building_land_type = building_land_type
        self.entry_date = entry_date
        self.economic_unit_id = economic_unit_id
        self.postcode = postcode
        self.town = town
        self.street = street
        self.house_number = house_number
        self.house_number_addition = house_number_addition
        self.country_id = country_id
        self.country = country
        self.street_complete = street_complete
        self.house_number_complete = house_number_complete
        self.construction_year = construction_year
        self.move_in_date = move_in_date
        self.building_type_id = building_type_id
        self.building_type_name = building_type_name
        self.district_id = district_id

    def __repr__(self):
        return f"Building {self.id_num}, {self.street_complete}, {self.postcode} {self.town}"


class UseUnit(Base):
    __tablename__ = "wowi_use_units"
    internal_id = Column("internal_id", Integer, primary_key=True)
    id_num = Column("id_num", String(50))
    company_id = Column("company_id", Integer)
    building_id = Column(Integer, ForeignKey("wowi_buildings.internal_id"))
    building = relationship('Building', back_populates='use_units')
    economic_unit_id = Column(Integer, ForeignKey("wowi_economic_units.internal_id"))
    economic_unit = relationship('EconomicUnit', back_populates='use_units')
    postcode = Column("postcode", String(30))
    town = Column("town", String(50))
    street = Column("street", String(100))
    house_number = Column("house_number", String(30))
    house_number_addition = Column("house_number_addition", String(30), nullable=True)
    country_id = Column("country_id", Integer)
    country = Column("country", String(50))
    street_complete = Column("street_complete", String(100))
    house_number_complete = Column("house_number_complete", String(30))
    financing_type_id = Column("financing_type_id", Integer)
    financing_type = Column("financing_type", String(50))
    use_unit_usage_type_id = Column("use_unit_usage_type_id", Integer)
    use_unit_usage_type = Column("use_unit_usage_type", String(50))
    usable_space = Column("usable_space", Float, nullable=True)
    living_space = Column("living_space", Float, nullable=True)
    heating_space = Column("heating_space", Float, nullable=True)
    number_of_rooms = Column("number_of_rooms", Integer, nullable=True)
    number_of_half_rooms = Column("number_of_half_rooms", Integer, nullable=True)
    description_of_position = Column("position_description", String(100), nullable=True)
    management_start = Column("management_start", Date, nullable=True)
    management_end = Column("management_end", Date, nullable=True)
    move_in_date = Column("move_in_date", Date, nullable=True)
    exit_date = Column("exit_date", Date, nullable=True)
    entry_date = Column("entry_date", Date, nullable=True)
    position_id = Column("position_id", Integer, nullable=True)
    position = Column("position", String(100), nullable=True)
    floor_id = Column("floor_id", Integer, nullable=True)
    floor_name = Column("floor_name", String(100), nullable=True)
    floor_level = Column("floor_level", Float, nullable=True)
    contracts = relationship('Contract', back_populates='use_unit')
    contractors = relationship('Contractor', back_populates='use_unit')

    def __init__(self, internal_id, id_num, company_id, building_id, economic_unit_id,
                 postcode, town, street, house_number, house_number_addition, country_id, country,
                 street_complete, house_number_complete, financing_type_id, financing_type,
                 use_unit_usage_type_id, use_unit_usage_type, usable_space, living_space, heating_space,
                 number_of_rooms, number_of_half_rooms, description_of_position, management_start,
                 management_end, move_in_date, exit_date, entry_date, position_id, position, floor_id,
                 floor_name, floor_level):
        self.internal_id = internal_id
        self.id_num = id_num
        self.company_id = company_id
        self.building_id = building_id
        self.economic_unit_id = economic_unit_id
        self.postcode = postcode
        self.town = town
        self.street = street
        self.house_number = house_number
        self.house_number_addition = house_number_addition
        self.country_id = country_id
        self.country = country
        self.street_complete = street_complete
        self.house_number_complete = house_number_complete
        self.financing_type_id = financing_type_id
        self.financing_type = financing_type
        self.use_unit_usage_type_id = use_unit_usage_type_id
        self.use_unit_usage_type = use_unit_usage_type
        self.usable_space = usable_space
        self.living_space = living_space
        self.heating_space = heating_space
        self.number_of_rooms = number_of_rooms
        self.number_of_half_rooms = number_of_half_rooms
        self.description_of_position = description_of_position
        self.management_start = management_start
        self.management_end = management_end
        self.move_in_date = move_in_date
        self.exit_date = exit_date
        self.entry_date = entry_date
        self.position_id = position_id
        self.position = position
        self.floor_id = floor_id
        self.floor_name = floor_name
        self.floor_level = floor_level

    def __repr__(self):
        return f"Use Unit {self.id_num}"


class Address(Base):
    __tablename__ = "wowi_addresses"
    internal_id = Column("internal_id", Integer, primary_key=True)
    postcode = Column("postcode", String(30))
    town = Column("town", String(50))
    street = Column("street", String(100), nullable=True)
    house_number = Column("house_number", String(30), nullable=True)
    house_number_addition = Column("house_number_addition", String(30), nullable=True)
    country_id = Column("country_id", Integer, nullable=True)
    country = Column("country", String(50), nullable=True)
    street_complete = Column("street_complete", String(100))
    house_number_complete = Column("house_number_complete", String(60))
    address_type_id = Column("address_type_id", Integer, nullable=True)
    address_type = Column("address_type", String(50), nullable=True)
    valid_from = Column("valid_from", Date)
    valid_to = Column("valid_to", Date, nullable=True)

    communications = relationship('Communication', back_populates='related_address')
    person_id = Column(Integer, ForeignKey("wowi_persons.internal_id"))
    person = relationship('Person', back_populates='addresses')

    def __init__(self, internal_id, postcode, town, street, house_number, house_number_addition, country_id, country,
                 street_complete, house_number_complete, address_type_id, address_type, valid_from, valid_to,
                 person_id):
        self.internal_id = internal_id
        self.postcode = postcode
        self.town = town
        self.street = street
        self.house_number = house_number
        self.house_number_addition = house_number_addition
        self.country_id = country_id
        self.country = country
        self.street_complete = street_complete
        self.house_number_complete = house_number_complete
        self.address_type_id = address_type_id
        self.address_type = address_type
        self.valid_from = valid_from
        self.valid_to = valid_to
        self.person_id = person_id

    def __repr__(self):
        return f"Address {self.street_complete}, {self.postcode} {self.town}"


class Communication(Base):
    __tablename__ = "wowi_communications"
    internal_id = Column("internal_id", Integer, primary_key=True)
    related_address_id = Column(Integer, ForeignKey("wowi_addresses.internal_id"), nullable=True)
    related_address = relationship('Address', back_populates='communications')
    content = Column("content", String(100))
    explanation = Column("explanation", String(100), nullable=True)
    communication_type_id = Column("communication_type_id", Integer)
    communication_type = Column("communication_type", String(50))
    person_id = Column(Integer, ForeignKey("wowi_persons.internal_id"))
    person = relationship('Person', back_populates='communications')

    def __init__(self, internal_id, related_address_id, content, explanation, communication_type_id,
                 communication_type, person_id):
        self.internal_id = internal_id
        self.related_address_id = related_address_id
        self.content = content
        self.explanation = explanation
        self.communication_type_id = communication_type_id
        self.communication_type = communication_type
        self.person_id = person_id

    def __repr__(self):
        return f"Communication Id {self.internal_id}: {self.content}"


class Person(Base):
    __tablename__ = "wowi_persons"
    internal_id = Column("internal_id", Integer, primary_key=True)
    id_num = Column("id_num", String(50))
    name = Column("name", String(60))
    short_name = Column("short_name", String(100))
    valid_from = Column("valid_from", Date)
    valid_to = Column("valid_to", Date, nullable=True)
    long_name_1 = Column("long_name_1", String(100), nullable=True)
    long_name_2 = Column("long_name_2", String(100), nullable=True)
    vat_id = Column("vat_id", String(30), nullable=True)
    commercial_register_number = Column("commercial_register_number", String(50), nullable=True)
    commercial_register_town = Column("commercial_register_town", String(30), nullable=True)
    first_name = Column("first_name", String(60), nullable=True)
    last_name = Column("last_name", String(60), nullable=True)
    birth_date = Column("birth_date", Date, nullable=True)
    gender_id = Column("gender_id", Integer, nullable=True)
    gender_name = Column("gender_name", String(30), nullable=True)
    is_natural_person = Column("is_natural_person", Boolean)

    addresses = relationship('Address', back_populates='person')
    communications = relationship('Communication', back_populates='person')
    contractors = relationship('Contractor', back_populates='person')

    def __init__(self, internal_id, id_num, name, short_name, valid_from, valid_to, long_name_1, long_name_2,
                 vat_id, commercial_register_number, commercial_register_town, first_name, last_name, birth_date,
                 gender_id, gender_name):
        self.internal_id = internal_id
        self.id_num = id_num
        self.name = name
        self.short_name = short_name
        self.valid_from = valid_from
        self.valid_to = valid_to
        self.long_name_1 = long_name_1
        self.long_name_2 = long_name_2
        self.vat_id = vat_id
        self.commercial_register_number = commercial_register_number
        self.commercial_register_town = commercial_register_town
        self.first_name = first_name
        self.last_name = last_name
        self.birth_date = birth_date
        self.gender_id = gender_id
        self.gender_name = gender_name

        if self.first_name is None and self.last_name is None:
            self.is_natural_person = False
        else:
            self.is_natural_person = True

    def __repr__(self):
        return f"Person IdNum {self.id_num} - {self.short_name}"


class Contract(Base):
    __tablename__ = "wowi_contracts"
    internal_id = Column("internal_id", Integer, primary_key=True)
    id_num = Column("id_num", String(30))
    use_unit_id = Column(Integer, ForeignKey("wowi_use_units.internal_id"))
    use_unit = relationship('UseUnit', back_populates='contracts')
    restriction_id = Column("restriction_id", Integer)
    restriction_name = Column("restriction_name", String(40))
    is_vacancy = Column("is_vacancy", Boolean)
    status_id = Column("status_id", Integer)
    status_name = Column("status_name", String(30))
    life_id = Column("life_id", Integer)
    life_name = Column("life_name", String(50))
    contract_start = Column("contract_start", Date)
    contract_end = Column("contract_end", Date, nullable=True)
    contractors = relationship('Contractor', back_populates='contract')

    def __init__(self, internal_id, id_num, use_unit_id, restriction_id, restriction_name, is_vacancy,
                 status_id, status_name, life_id, life_name, contract_start, contract_end):
        self.internal_id = internal_id
        self.id_num = id_num
        self.use_unit_id = use_unit_id
        self.restriction_id = restriction_id
        self.restriction_name = restriction_name
        self.is_vacancy = is_vacancy
        self.status_id = status_id
        self.status_name = status_name
        self.life_id = life_id
        self.life_name = life_name
        self.contract_start = contract_start
        self.contract_end = contract_end

    def __repr__(self):
        return f"Contract IdNum {self.id_num} Id {self.internal_id}"


class Contractor(Base):
    __tablename__ = "wowi_contractors"
    internal_id = Column("internal_id", Integer, primary_key=True)
    contract_id = Column(Integer, ForeignKey("wowi_contracts.internal_id"), primary_key=True)
    type_id = Column("type_id", Integer, primary_key=True)
    person_id = Column(Integer, ForeignKey("wowi_persons.internal_id"))
    contract = relationship('Contract', back_populates='contractors')
    use_unit_id = Column(Integer, ForeignKey("wowi_use_units.internal_id"))
    use_unit = relationship('UseUnit', back_populates='contractors')
    person = relationship('Person', back_populates='contractors')

    type_name = Column("type_name", String(50))
    valid_from = Column("valid_from", Date)
    valid_to = Column("valid_to", Date, nullable=True)

    def __init__(self, internal_id, contract_id, use_unit_id, person_id, type_id,
                 type_name, valid_from, valid_to):
        self.internal_id = internal_id
        self.contract_id = contract_id
        self.use_unit_id = use_unit_id
        self.person_id = person_id
        self.type_id = type_id
        self.type_name = type_name
        self.valid_from = valid_from
        self.valid_to = valid_to

    def __repr__(self):
        return f"Contractor {self.internal_id}"



from setuptools import setup

setup(
    name='wowicache',
    version='1.0.14',
    description='OPENWOWI Wowiport SQLAlchemy overlay',
    url='https://github.com/seb-bau/wowicache',
    author='Sebastian Bauhaus',
    author_email='sebastian@bytewish.de',
    license='GPL-3.0',
    packages=['wowicache'],
    install_requires=[
        'attrs>=23.1.0',
        'cattrs>=23.1.2',
        'certifi>=2023.7.22',
        'charset-normalizer>=3.2.0',
        'graypy>=2.1.0',
        'idna>=3.4',
        'jsonmerge>=1.9.2',
        'jsonschema>=4.19.0',
        'jsonschema-specifications>=2023.7.1',
        'platformdirs>=3.10.0',
        'pyhumps>=3.8.0',
        'python-dotenv>=1.0.0',
        'referencing>=0.30.2',
        'requests>=2.31.0',
        'requests-cache>=1.1.0',
        'rpds-py>=0.9.2',
        'six>=1.16.0',
        'url-normalize>=1.4.3',
        'urllib3>=2.0.4',
        'wowipy>=1.1.14',
        'SQLAlchemy<2'
                      ],

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12'
    ],
)
